<?php
/**
 * Plugin Name: Wishlists for WooCommerce by Iconic
 * Plugin URI: https://iconicwp.com/products/wishlists-for-woocommerce/
 * Description: Enable wishlists for your customers.
 * Version: 1.0.2
 * Author: Iconic
 * Author URI: https://iconicwp.com
 * Text Domain: iconic-ww
 * WC requires at least: 3.4.2
 * WC tested up to: 3.5.5
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Iconic_Woo_Wishlists {
	/**
	 * Plugin version.
	 *
	 * @var string
	 */
	public static $version = "1.0.2";

	/**
	 * Construct the plugin
	 */
	public function __construct() {
		load_plugin_textdomain( 'iconic-ww', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

		$this->define_constants();
		$this->load_classes();

		if ( ! Iconic_WW_Core_Helpers::is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			return;
		}

	}

	/**
	 * Run on activation.
	 */
	public static function activate() {
		if ( ! get_option( 'iconic_ww_flush_rewrite_rules_flag' ) ) {
			add_option( 'iconic_ww_flush_rewrite_rules_flag', true );
		}
	}

	/**
	 * Define Constants.
	 */
	private function define_constants() {
		$this->define( 'ICONIC_WW_PATH', plugin_dir_path( __FILE__ ) );
		$this->define( 'ICONIC_WW_URL', plugin_dir_url( __FILE__ ) );
		$this->define( 'ICONIC_WW_INC_PATH', ICONIC_WW_PATH . 'inc/' );
		$this->define( 'ICONIC_WW_VENDOR_PATH', ICONIC_WW_INC_PATH . 'vendor/' );
		$this->define( 'ICONIC_WW_TPL_PATH', ICONIC_WW_PATH . 'templates/' );
		$this->define( 'ICONIC_WW_BASENAME', plugin_basename( __FILE__ ) );
		$this->define( 'ICONIC_WW_VERSION', self::$version );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @param string      $name
	 * @param string|bool $value
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Load classes
	 */
	private function load_classes() {
		require_once( ICONIC_WW_INC_PATH . 'class-core-autoloader.php' );

		Iconic_WW_Core_Autoloader::run( array(
			'prefix'   => 'Iconic_WW_',
			'inc_path' => ICONIC_WW_INC_PATH,
		) );

		

		Iconic_WW_Core_Settings::run( array(
			'parent_slug'   => 'edit.php?post_type=iconic_ww_wishlist',
			'vendor_path'   => ICONIC_WW_VENDOR_PATH,
			'title'         => 'Wishlists for WooCommerce',
			'version'       => self::$version,
			'menu_title'    => __( 'Settings', 'iconic-ww' ),
			'settings_path' => ICONIC_WW_INC_PATH . 'admin/settings.php',
			'option_group'  => 'iconic_ww',
			'docs'          => array(
				'collection'      => '/collection/213-wishlists-for-woocommerce',
				'troubleshooting' => '/category/219-troubleshooting',
				'getting-started' => '/category/216-getting-started',
			),
			'cross_sells'   => array(
				'iconic-woo-show-single-variations',
				'iconic-woothumbs',
			),
		) );


		Iconic_WW_Settings::run();
		Iconic_WW_Assets::run();
		Iconic_WW_Wishlists::run();
		Iconic_WW_Ajax::run();
		Iconic_WW_Shortcodes::run();
	}
}

$iconic_ww = new Iconic_Woo_Wishlists();

register_activation_hook( __FILE__, array( $iconic_ww, 'activate' ) );