<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Class Iconic_WW_Ajax
 */
class Iconic_WW_Ajax {
	/**
	 * Init
	 */
	public static function run() {
		self::add_ajax_events();
	}

	/**
	 * Hook in methods - uses WordPress ajax handlers (admin-ajax).
	 */
	public static function add_ajax_events() {
		// iconic_ww_{event} => nopriv
		$ajax_events = array(
			'get_add_to_wishlist_modal' => true,
			'get_create_wishlist_modal' => true,
			'get_edit_wishlist_modal'   => true,
			'get_delete_wishlist_modal' => true,
			'get_share_wishlist_modal'  => true,
			'add_to_wishlist'           => false,
			'edit_wishlist'             => false,
			'delete_wishlist'           => false,
			'remove_product'            => false,
		);

		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_iconic_ww_' . $ajax_event, array( __CLASS__, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_iconic_ww_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			}
		}
	}

	/**
	 * Get add to wishlist modal.
	 */
	public static function get_add_to_wishlist_modal() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'request_id'   => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'product_id'   => absint( filter_input( INPUT_POST, 'product_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'variation_id' => absint( filter_input( INPUT_POST, 'variation_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'attributes'   => isset( $_POST['attributes'] ) ? (array) $_POST['attributes'] : '',
		);

		if ( ! empty( $data['product_id'] ) ) {
			$data['product'] = wc_get_product( $data['product_id'] );
		}

		$modal         = new Iconic_WW_Modal( 'add-to-wishlist', $data );
		$data['modal'] = $modal->get_output();

		wp_send_json_success( $data );
	}

	/**
	 * Get create wishlist modal.
	 */
	public static function get_create_wishlist_modal() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'request_id' => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		$modal         = new Iconic_WW_Modal( 'create-wishlist', $data );
		$data['modal'] = $modal->get_output();

		wp_send_json_success( $data );
	}

	/**
	 * Get edit wishlist modal.
	 */
	public static function get_edit_wishlist_modal() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'request_id'  => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'wishlist_id' => absint( filter_input( INPUT_POST, 'wishlist_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		$modal         = new Iconic_WW_Modal( 'edit-wishlist', $data );
		$data['modal'] = $modal->get_output();

		wp_send_json_success( $data );
	}

	/**
	 * Get delete wishlist modal.
	 */
	public static function get_delete_wishlist_modal() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'request_id'  => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'wishlist_id' => absint( filter_input( INPUT_POST, 'wishlist_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		$modal         = new Iconic_WW_Modal( 'delete-wishlist', $data );
		$data['modal'] = $modal->get_output();

		wp_send_json_success( $data );
	}

	/**
	 * Get share wishlist modal.
	 */
	public static function get_share_wishlist_modal() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'request_id'  => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'wishlist_id' => absint( filter_input( INPUT_POST, 'wishlist_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		$modal         = new Iconic_WW_Modal( 'share-wishlist', $data );
		$data['modal'] = $modal->get_output();

		wp_send_json_success( $data );
	}

	/**
	 * Add item to wishlist.
	 */
	public static function add_to_wishlist() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$default_error = __( 'An error occurred when adding that item. Please try again.', 'iconic-ww' );

		$data = array(
			'request_id' => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		$post_data = Iconic_WW_Helpers::get_formatted_form_data( $_POST['data'] );

		if ( empty( $post_data ) || empty( $post_data['product_id'] ) ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		$product = wc_get_product( $post_data['product_id'] );

		if ( ! $product ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		$type = $post_data['choose_wishlist'] === 'iconic_ww_new' ? 'new' : 'existing';

		if ( $type === 'existing' ) {
			if ( empty( $post_data['choose_wishlist'] ) ) {
				$data['message'] = sprintf( __( 'Please choose an existing %s.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
				wp_send_json_error( $data );
			}

			$wishlist_id = $post_data['choose_wishlist'];
		} else {
			if ( empty( $post_data['wishlist_name'] ) ) {
				$data['message'] = sprintf( __( 'Please enter a name for your %s.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
				wp_send_json_error( $data );
			}

			$wishlist_id = Iconic_WW_Wishlists::create_wishlist( array(
				'post_title'  => $post_data['wishlist_name'],
				'post_status' => $post_data['wishlist_privacy'],
			) );
		}

		$wishlist   = new Iconic_WW_Wishlist( $wishlist_id );
		$attributes = is_array( $post_data['attributes'] ) ? $post_data['attributes'] : json_decode( stripslashes( $post_data['attributes'] ), true );
		$add_item   = $wishlist->add_item( $post_data['product_id'], $post_data['variation_id'], $attributes );

		if ( ! $add_item ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		if ( $add_item === 'exists' ) {
			$name            = ! empty( $attributes ) ? sprintf( __( 'that variation of %s', 'iconic-ww' ), $product->get_name() ) : $product->get_name();
			$data['message'] = sprintf( __( 'You have already added %s to "%s".', 'iconic-ww' ), $name, $wishlist->get( 'post_title' ) );
			wp_send_json_error( $data );
		}

		$link            = sprintf( '<a href="%s">%s</a>', $wishlist->get( 'url' ), $wishlist->get( 'post_title' ) );
		$data['message'] = sprintf( __( '%s was successfully added to %s.', 'iconic-ww' ), $product->get_name(), $link );

		wp_send_json_success( $data );
	}

	/**
	 * Create wishlist.
	 */
	public static function edit_wishlist() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$default_error = sprintf( __( 'An error occurred while creating the %s. Please try again.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );

		$data = array(
			'request_id' => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'wishlist'   => false,
		);

		$data['form_data'] = Iconic_WW_Helpers::get_formatted_form_data( $_POST['data'] );

		if ( empty( $data['form_data'] ) ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		if ( empty( $data['form_data']['wishlist_name'] ) ) {
			$data['message'] = sprintf( __( 'Please enter a title for the %s.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
			wp_send_json_error( $data );
		}

		$args = array(
			'post_title'  => $data['form_data']['wishlist_name'],
			'post_status' => empty( $data['form_data']['wishlist_privacy'] ) ? 'publish' : $data['form_data']['wishlist_privacy'],
		);

		if ( $data['form_data']['edit_type'] === 'create' ) {
			$wishlist_id = Iconic_WW_Wishlists::create_wishlist( $args );

			if ( ! $wishlist_id ) {
				$data['message'] = $default_error;
				wp_send_json_error( $data );
			}

			$data['wishlist']['id'] = $wishlist_id;
			$data['message']        = sprintf( __( 'Your %s was created successfully.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
		} else {
			if ( empty( $data['form_data']['wishlist_id'] ) ) {
				$data['message'] = sprintf( __( 'There was an issue updating your %s. Please try again.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
			} else {
				$args['ID'] = $data['form_data']['wishlist_id'];
				$update     = Iconic_WW_Wishlists::update_wishlist( $args );

				if ( $update === 0 ) {
					$data['message'] = sprintf( __( 'There was an issue updating your %s. Please try again.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
				} else {
					$data['wishlist']['id'] = $data['form_data']['wishlist_id'];
					$data['message']        = sprintf( __( 'Your %s has been updated.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );
				}
			}
		}

		if ( ! empty( $data['wishlist']['id'] ) ) {
			$wishlist         = new Iconic_WW_Wishlist( $data['wishlist']['id'] );
			$data['wishlist'] = $wishlist->get( 'info' );

			if ( isset( $data['form_data']['wishlist_user_name'] ) ) {
				$wishlist->set( 'meta', array(
					'key'   => 'user_name',
					'value' => $data['form_data']['wishlist_user_name'],
				) );
			}

			if ( isset( $data['form_data']['wishlist_email'] ) ) {
				$wishlist->set( 'meta', array(
					'key'   => 'email',
					'value' => $data['form_data']['wishlist_email'],
				) );
			}
		}

		wp_send_json_success( $data );
	}

	/**
	 * Delete wishlist.
	 */
	public static function delete_wishlist() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$default_error = sprintf( __( 'An error occurred when deleting the %s. Please try again.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );

		$data = array(
			'request_id'    => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'has_wishlists' => true,
		);

		$data['form_data'] = Iconic_WW_Helpers::get_formatted_form_data( $_POST['data'] );

		if ( empty( $data['form_data'] ) || empty( $data['form_data']['wishlist_id'] ) ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		$delete = wp_delete_post( $data['form_data']['wishlist_id'], true );

		if ( ! $delete ) {
			$data['message'] = $default_error;
			wp_send_json_error( $data );
		}

		$user                  = new Iconic_WW_User();
		$data['has_wishlists'] = $user->has_wishlists();
		$data['message']       = sprintf( __( 'Your %s was deleted successfully.', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular', true ) );

		wp_send_json_success( $data );
	}

	/**
	 * Remove product from wishlist.
	 */
	public static function remove_product() {
		check_ajax_referer( 'iconic-ww', 'nonce' );

		$data = array(
			'message'     => __( 'An error occurred when removing the product. Please try again.', 'iconic-ww' ),
			'request_id'  => absint( filter_input( INPUT_POST, 'request_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'product_id'  => absint( filter_input( INPUT_POST, 'product_id', FILTER_SANITIZE_NUMBER_INT ) ),
			'wishlist_id' => absint( filter_input( INPUT_POST, 'wishlist_id', FILTER_SANITIZE_NUMBER_INT ) ),
		);

		if ( empty( $data['product_id'] ) || empty( $data['wishlist_id'] ) ) {
			wp_send_json_error( $data );
		}

		$wishlist = new Iconic_WW_Wishlist( $data['wishlist_id'] );

		if ( ! $wishlist ) {
			wp_send_json_error( $data );
		}

		$delete = $wishlist->remove_item( $data['product_id'] );

		if ( ! $delete ) {
			wp_send_json_error( $data );
		}

		$product_title   = get_the_title( $data['product_id'] );
		$data['message'] = sprintf( __( '%s was removed successfully.', 'iconic-ww' ), $product_title );

		wp_send_json_success( $data );
	}
}