<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_Products.
 *
 * @class    Iconic_WW_Products
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_Products {
	/**
	 * Run.
	 */
	public static function run() {
		add_action( 'init', array( __CLASS__, 'maybe_insert_buttons' ) );
		add_filter( 'woocommerce_get_item_data', array( __CLASS__, 'get_item_data' ), 10, 2 );
		add_filter( 'woocommerce_loop_add_to_cart_link', array( __CLASS__, 'loop_add_to_cart_link' ), 10, 3 );
		add_filter( 'woocommerce_product_variation_get_attributes', array( __CLASS__, 'get_variation_add_to_cart_attributes' ), 10, 2 );
		add_filter( 'woocommerce_cart_item_data_to_validate', array( __CLASS__, 'cart_item_data_to_validate' ), 10, 2 );
	}

	/**
	 * Insert buttons at the right position.
	 */
	public static function maybe_insert_buttons() {
		$buttons = array(
			'buttons_positions_add_to_wishlist',
			'buttons_positions_add_to_wishlist_shop',
		);

		foreach ( $buttons as $button ) {
			$position = Iconic_WW_Core_Settings::get_setting( $button );

			if ( empty( $position ) ) {
				continue;
			}

			$position_data = self::get_position_data( $position );

			if ( ! $position_data ) {
				continue;
			}

			add_action( $position_data['position'], array( __CLASS__, 'add_to_wishlist_button' ), $position_data['priority'] );
		}
	}

	/**
	 * Get position data.
	 *
	 * @param $position
	 *
	 * @return bool|mixed
	 */
	public static function get_position_data( $position ) {
		$data = apply_filters( 'iconic_ww_position_data', array(
			'after_add_to_cart'      => array(
				'position' => 'woocommerce_single_product_summary',
				'priority' => 35,
			),
			'after_title'            => array(
				'position' => 'woocommerce_single_product_summary',
				'priority' => 15,
			),
			'before_description'     => array(
				'position' => 'woocommerce_single_product_summary',
				'priority' => 15,
			),
			'after_description'      => array(
				'position' => 'woocommerce_single_product_summary',
				'priority' => 25,
			),
			'before_link_shop'       => array(
				'position' => 'woocommerce_before_shop_loop_item',
				'priority' => 1,
			),
			'after_thumbnail_shop'   => array(
				'position' => 'woocommerce_before_shop_loop_item_title',
				'priority' => 15,
			),
			'after_title_shop'       => array(
				'position' => 'woocommerce_shop_loop_item_title',
				'priority' => 15,
			),
			'after_add_to_cart_shop' => array(
				'position' => 'woocommerce_after_shop_loop_item',
				'priority' => 15,
			),
		), $position );

		if ( ! isset( $data[ $position ] ) ) {
			return false;
		}

		return $data[ $position ];
	}

	/**
	 * Add to wishlist button.
	 */
	public static function add_to_wishlist_button() {
		Iconic_WW_Helpers::include_template( 'components/add-to-wishlist-button' );
	}

	/**
	 * Get item data for variable products too.
	 *
	 * @param $item_data
	 * @param $cart_item
	 *
	 * @return array
	 */
	public static function get_item_data( $item_data, $cart_item ) {
		if ( ! isset( $cart_item['iconic-ww'] ) ) {
			return $item_data;
		}

		if ( empty( $cart_item['variation'] ) || ! $cart_item['data']->is_type( 'variable' ) ) {
			return $item_data;
		}

		foreach ( $cart_item['variation'] as $name => $value ) {
			$taxonomy = wc_attribute_taxonomy_name( str_replace( 'attribute_pa_', '', urldecode( $name ) ) );

			if ( taxonomy_exists( $taxonomy ) ) {
				// If this is a term slug, get the term's nice name.
				$term = get_term_by( 'slug', $value, $taxonomy );
				if ( ! is_wp_error( $term ) && $term && $term->name ) {
					$value = $term->name;
				}
				$label = wc_attribute_label( $taxonomy );
			} else {
				// If this is a custom option slug, get the options name.
				$value = apply_filters( 'woocommerce_variation_option_name', $value );
				$label = wc_attribute_label( str_replace( 'attribute_', '', $name ), $cart_item['data'] );
			}

			// Check the nicename against the title.
			if ( '' === $value || wc_is_attribute_in_product_name( $value, $cart_item['data']->get_name() ) ) {
				continue;
			}

			$item_data[] = array(
				'key'   => $label,
				'value' => $value,
			);
		}

		return $item_data;
	}

	/**
	 * Get add to cart link.
	 *
	 * @param string     $link
	 * @param WC_Product $product
	 * @param array      $args
	 *
	 * @return string
	 */
	public static function loop_add_to_cart_link( $link, $product, $args ) {
		if ( ! isset( $args['iconic-ww'] ) ) {
			return $link;
		}

		$attributes = isset( $args['iconic-ww-attributes'] ) ? $args['iconic-ww-attributes'] : array();

		if ( empty( $attributes ) ) {
			return $link;
		}

		if ( $product->is_type( 'variation' ) ) {
			$args['attributes']['data-iconic_ww_attributes'] = json_encode( $attributes );
		}

		$add_to_cart_url = $product->add_to_cart_url();
		$add_to_cart_url = Iconic_WW_Helpers::get_permalink( $product, $attributes, $add_to_cart_url );

		return sprintf( '<a href="%s" data-quantity="%s" class="%s" %s>%s</a>',
			esc_url( $add_to_cart_url ),
			esc_attr( isset( $args['quantity'] ) ? $args['quantity'] : 1 ),
			esc_attr( isset( $args['class'] ) ? $args['class'] : 'button' ),
			isset( $args['attributes'] ) ? wc_implode_html_attributes( $args['attributes'] ) : '',
			esc_html( $product->add_to_cart_text() )
		);
	}

	/**
	 * Get variation attributes when adding to cart from wishlist.
	 *
	 * @param mixed   $value
	 * @param WC_Data $data
	 *
	 * @return mixed
	 */
	public static function get_variation_add_to_cart_attributes( $value, $data ) {
		if ( empty( $_POST['iconic_ww_attributes'] ) ) {
			return $value;
		}

		$attributes = array();

		foreach ( $_POST['iconic_ww_attributes'] as $key => $value ) {
			$key                = str_replace( 'attribute_', '', $key );
			$attributes[ $key ] = $value;
		}

		return $attributes;
	}

	/**
	 * Make sure our attributes aren't added when validating the product
	 * in the cart. They are only required in order to add the item
	 * during the initial add to cart request.
	 *
	 * @param array      $data
	 * @param WC_Product $product
	 *
	 * @return mixed
	 */
	public static function cart_item_data_to_validate( $data, $product ) {
		$ajax = filter_input( INPUT_GET, 'wc-ajax' );

		if ( 'add_to_cart' !== $ajax || ! $product->is_type( 'variation' ) ) {
			return $data;
		}

		remove_filter( 'woocommerce_product_variation_get_attributes', array( __CLASS__, 'get_variation_add_to_cart_attributes' ), 10 );

		$data['attributes'] = $product->get_variation_attributes();

		add_filter( 'woocommerce_product_variation_get_attributes', array( __CLASS__, 'get_variation_add_to_cart_attributes' ), 10, 2 );

		return $data;
	}
}