<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_Settings.
 *
 * @class    Iconic_WW_Settings
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_Settings {
	/**
	 * Run.
	 */
	public static function run() {
		add_action( "update_option_iconic_ww_settings", array( __CLASS__, 'on_save' ), 10, 2 );
	}

	/**
	 * Setup temporary admin menu.
	 */
	public static function setup_admin_menu() {
		add_action( 'admin_menu', array( __CLASS__, 'add_menu_item' ) );
	}

	/**
	 * Add temporary Wishlists menu item.
	 *
	 * This is required so the licence can be entered before
	 * the post type is initiated.
	 */
	public static function add_menu_item() {
		if ( Iconic_WW_Core_Licence::has_valid_licence() ) {
			return;
		}

		$label = Iconic_WW_Wishlists::get_wishlists_label( 'plural' );

		add_menu_page(
			$label,
			$label,
			'manage_options',
			'iconic-ww-settings',
			'',
			'dashicons-heart',
			5
		);
	}

	/**
	 * Run actions on save settings.
	 *
	 * @param $old
	 * @param $new
	 */
	public static function on_save( $old, $new ) {
		if ( $old['general_page_id'] !== $new['general_page_id'] ) {
			flush_rewrite_rules();
		}
	}

	/**
	 * @param $icon
	 *
	 * @return mixed
	 */
	public static function get_icon_name( $icon ) {
		return Iconic_WW_Core_Settings::get_setting( 'buttons_style_' . $icon . '_icon' );
	}

	/**
	 * Output icon.
	 *
	 * @param $icon
	 * @param $size
	 */
	public static function output_icon( $icon, $size = false ) {
		$name = self::get_icon_name( $icon );
		Iconic_WW_Helpers::output_svg( $name, $size );
	}
}