<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_Wishlists.
 *
 * @class    Iconic_WW_Wishlists
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_Wishlists {
	/**
	 * Post type.
	 *
	 * @var string
	 */
	public static $post_type = 'iconic_ww_wishlist';

	/**
	 * Run.
	 */
	public static function run() {
		Iconic_WW_Endpoints::run();
		Iconic_WW_Database::run();
		Iconic_WW_Products::run();

		// Global hooks.
		add_action( 'init', array( __CLASS__, 'setup_post_type' ), 0 );
		add_action( 'save_post', array( __CLASS__, 'set_slug' ) );

		if ( ! is_admin() ) {
			// Frontend hooks.
			add_filter( 'posts_join', array( __CLASS__, 'search_join' ), 10, 2 );
			add_filter( 'posts_where', array( __CLASS__, 'search_where' ), 10, 2 );
			add_filter( 'posts_distinct', array( __CLASS__, 'search_distinct' ) );
		} else {
			// Admin hooks.
			add_filter( 'manage_' . self::$post_type . '_posts_columns', array( __CLASS__, 'admin_columns' ) );
			add_action( 'manage_' . self::$post_type . '_posts_custom_column', array( __CLASS__, 'admin_columns_content' ), 10, 2 );
			add_action( 'add_meta_boxes', array( __CLASS__, 'meta_boxes' ) );
			add_action( 'load-post.php', array( __CLASS__, 'load_user_dropdown_filter' ) );
			add_action( 'load-post-new.php', array( __CLASS__, 'load_user_dropdown_filter' ) );
		}
	}

	/**
	 * Get wishlists label.
	 *
	 * @return string|array
	 */
	public static function get_wishlists_label( $type = false, $lower = false ) {
		$labels = apply_filters( 'iconic_ww_wishlists_labels', array(
			'plural'   => __( 'Wishlists', 'iconic-ww' ),
			'singular' => __( 'Wishlist', 'iconic-ww' ),
		) );

		if ( ! $type ) {
			return $labels;
		}

		if ( ! isset( $labels[ $type ] ) ) {
			return '';
		}

		return $lower ? strtolower( $labels[ $type ] ) : $labels[ $type ];
	}

	/**
	 * Add post type.
	 */
	public static function setup_post_type() {
		$endpoints        = Iconic_WW_Endpoints::get_endpoints();
		$wishlists_labels = self::get_wishlists_label();

		$labels = array(
			'name'                  => $wishlists_labels['plural'],
			'singular_name'         => $wishlists_labels['plural'],
			'menu_name'             => $wishlists_labels['plural'],
			'name_admin_bar'        => $wishlists_labels['singular'],
			'archives'              => sprintf( __( '%s Archives', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'attributes'            => sprintf( __( '%s Attributes', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'parent_item_colon'     => sprintf( __( 'Parent %s:', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'all_items'             => sprintf( __( 'All %s', 'iconic-ww' ), $wishlists_labels['plural'] ),
			'add_new_item'          => sprintf( __( 'Add New %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'add_new'               => __( 'Add New', 'iconic-ww' ),
			'new_item'              => sprintf( __( 'New %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'edit_item'             => sprintf( __( 'Edit %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'update_item'           => sprintf( __( 'Update %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'view_item'             => sprintf( __( 'View %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'view_items'            => sprintf( __( 'View %s', 'iconic-ww' ), $wishlists_labels['plural'] ),
			'search_items'          => sprintf( __( 'Search %s', 'iconic-ww' ), $wishlists_labels['plural'] ),
			'not_found'             => __( 'Not found', 'iconic-ww' ),
			'not_found_in_trash'    => __( 'Not found in Trash', 'iconic-ww' ),
			'featured_image'        => __( 'Featured Image', 'iconic-ww' ),
			'set_featured_image'    => __( 'Set featured image', 'iconic-ww' ),
			'remove_featured_image' => __( 'Remove featured image', 'iconic-ww' ),
			'use_featured_image'    => __( 'Use as featured image', 'iconic-ww' ),
			'uploaded_to_this_item' => sprintf( __( 'Uploaded to this %s', 'iconic-ww' ), $wishlists_labels['singular'] ),
			'items_list'            => $wishlists_labels['plural'],
			'items_list_navigation' => sprintf( __( '%s navigation', 'iconic-ww' ), $wishlists_labels['plural'] ),
			'filter_items_list'     => sprintf( __( 'Filter %s', 'iconic-ww' ), $wishlists_labels['plural'] ),
		);

		$args = array(
			'label'               => $wishlists_labels['singular'],
			'labels'              => $labels,
			'supports'            => array( 'title', 'author' ),
			'hierarchical'        => false,
			'public'              => false,
			'show_ui'             => true,
			'show_in_menu'        => true,
			'menu_position'       => 5,
			'menu_icon'           => 'dashicons-heart',
			'show_in_admin_bar'   => false,
			'show_in_nav_menus'   => false,
			'can_export'          => true,
			'has_archive'         => false,
			'exclude_from_search' => true,
			'publicly_queryable'  => true,
			'query_var'           => 'wishlists',
			'rewrite'             => false,
			'capability_type'     => 'post',
			'show_in_rest'        => true,
			'rest_base'           => 'wishlists',
		);

		register_post_type( self::$post_type, $args );

		if ( $endpoints && get_option( 'iconic_ww_flush_rewrite_rules_flag' ) ) {
			flush_rewrite_rules();
			delete_option( 'iconic_ww_flush_rewrite_rules_flag' );
		}
	}

	/**
	 * Set unique slug.
	 *
	 * @param int $post_id
	 */
	public static function set_slug( $post_id ) {
		if ( get_post_type( $post_id ) !== self::$post_type ) {
			return;
		}

		if ( wp_is_post_revision( $post_id ) ) {
			return;
		}

		$slug         = self::generate_slug( $post_id );
		$current_slug = get_post_field( 'post_name', $post_id );

		if ( $slug === $current_slug ) {
			return;
		}

		// unhook this function to prevent infinite looping
		remove_action( 'save_post', array( __CLASS__, 'set_slug' ) );

		// update the post slug
		wp_update_post( array(
			'ID'        => $post_id,
			'post_name' => $slug,
		) );

		// re-hook this function
		add_action( 'save_post', array( __CLASS__, 'set_slug' ) );
	}

	/**
	 * Generate unique slug.
	 *
	 * @param int      $post_id
	 * @param null|int $user_id
	 *
	 * @return string
	 */
	public static function generate_slug( $post_id, $user_id = null ) {
		$current_user_id = $user_id ? $user_id : get_current_user_id();
		$slug            = sprintf( '%d-%d', $current_user_id, $post_id );

		return apply_filters( 'iconic_ww_slug', substr( md5( $slug ), 0, 6 ), $post_id );
	}

	/**
	 * Maybe create default list if it doesn't exist.
	 */
	public static function maybe_create_default_wishlist() {
		$current_user_id = get_current_user_id();

		if ( ! $current_user_id ) {
			return;
		}

		$user            = new Iconic_WW_User( $current_user_id );
		$default_list_id = $user->get( 'default_wishlist_id' );

		if ( $default_list_id ) {
			return;
		}

		$wishlist_id = self::create_wishlist( array(
			'post_author' => $current_user_id,
			'post_title'  => sprintf( '%s %s', __( 'Default', 'iconic-ww' ), self::get_wishlists_label( 'singular' ) ),
		) );

		if ( ! $wishlist_id ) {
			return;
		}

		$user->set( 'default_wishlist_id', array( 'id' => $wishlist_id ) );

		do_action( 'iconic_ww_default_wishlist_created', $wishlist_id, $user );
	}

	/**
	 * Create wishlist.
	 *
	 * @param array $args
	 *
	 * @return bool|int
	 */
	public static function create_wishlist( $args = array() ) {
		if ( empty( $args['post_author'] ) ) {
			$args['post_author'] = get_current_user_id();
		}

		$args = array_filter( $args );

		$defaults = array(
			'post_title'  => self::get_wishlists_label( 'singular' ),
			'post_type'   => self::$post_type,
			'post_status' => 'publish',
		);

		$args = wp_parse_args( $args, $defaults );
		$args = apply_filters( 'iconic_ww_create_wishlist_args', $args );

		$insert = wp_insert_post( $args );

		if ( is_wp_error( $insert ) ) {
			return false;
		}

		do_action( 'iconic_ww_wishlist_created', $insert, $args );

		return $insert;
	}

	/**
	 * Update wishlist.
	 *
	 * @param array $args
	 *
	 * @return bool|int
	 */
	public static function update_wishlist( $args = array() ) {
		$args = apply_filters( 'iconic_ww_update_wishlist_args', $args );

		if ( empty( $args['ID'] ) ) {
			return false;
		}

		$update = wp_update_post( $args );

		do_action( 'iconic_ww_wishlist_updated', $update, $args );

		return $update;
	}

	/**
	 * Get Wishlists.
	 *
	 * @param array $args
	 *
	 * @return array
	 */
	public static function get_wishlists( $args = array() ) {
		$wishlists = array();

		$paged = get_query_var( 'paged' );

		$defaults = array(
			'post_type'   => self::$post_type,
			'author'      => get_current_user_id(),
			'orderby'     => 'date',
			'order'       => 'DESC',
			'post_status' => array( 'publish', 'private' ),
			'paged'       => $paged ? $paged : 1,
		);

		$args = wp_parse_args( $args, $defaults );

		$s = filter_input( INPUT_GET, 'search' );

		if ( $s ) {
			$args['s'] = $s;
		}

		$args = apply_filters( 'iconic_ww_get_wishlists_args', $args );

		$GLOBALS['iconic_ww_wishlists_query'] = new WP_Query( $args );

		if ( ! $GLOBALS['iconic_ww_wishlists_query']->have_posts() ) {
			return $wishlists;
		}

		foreach ( $GLOBALS['iconic_ww_wishlists_query']->posts as $wishlist ) {
			$wishlists[] = new Iconic_WW_Wishlist( $wishlist );
		}

		return $wishlists;
	}

	/**
	 * Add meta to search.
	 *
	 * @param $join
	 * @param $query
	 *
	 * @return string
	 */
	public static function search_join( $join, $query ) {
		if ( ! Iconic_WW_Endpoints::is( 'find' ) || ! isset( $_GET['search'] ) ) {
			return $join;
		}

		global $wpdb;

		$join .= " LEFT JOIN {$wpdb->postmeta} iconic_ww ON {$wpdb->posts}.ID = iconic_ww.post_id ";

		return $join;
	}

	/**
	 * Add name and email to search.
	 *
	 * @param $where
	 * @param $query
	 *
	 * @return mixed
	 */
	public static function search_where( $where, $query ) {
		if ( ! Iconic_WW_Endpoints::is( 'find' ) || ! isset( $_GET['search'] ) ) {
			return $where;
		}

		global $wpdb;

		$where = preg_replace(
			"/\(\s*{$wpdb->posts}.post_title\s+LIKE\s*(\'[^\']+\')\s*\)/",
			"({$wpdb->posts}.post_title LIKE $1) 
			OR (iconic_ww.meta_key = 'iconic_ww_user_name' AND iconic_ww.meta_value LIKE $1)
			OR (iconic_ww.meta_key = 'iconic_ww_email' AND iconic_ww.meta_value LIKE $1)", $where );

		return $where;
	}

	/**
	 * Make sure duplicates aren't returned.
	 *
	 * @param $where
	 *
	 * @return string
	 */
	public static function search_distinct( $where ) {
		if ( ! Iconic_WW_Endpoints::is( 'find' ) || ! isset( $_GET['search'] ) ) {
			return $where;
		}

		return "DISTINCT";
	}

	/**
	 * Get wishlists base slug.
	 *
	 * @return string
	 */
	public static function get_wishlists_base_slug() {
		$page_id = self::get_wishlists_base_page_id();
		$uri     = $page_id ? get_page_uri( $page_id ) : false;

		if ( ! $uri ) {
			return false;
		}

		return $uri;
	}

	/**
	 * Get wishlists base page ID.
	 *
	 * @return int
	 */
	public static function get_wishlists_base_page_id() {
		static $id = null;

		if ( ! is_null( $id ) ) {
			return $id;
		}

		$setting = Iconic_WW_Core_Settings::get_setting( 'general_page_id' );
		$id      = empty( $setting ) ? $id : absint( $setting );

		return $id;
	}

	/**
	 * Get statuses.
	 *
	 * @param array $params
	 *
	 * @return array
	 */
	public static function get_statuses( $params = array() ) {
		return apply_filters( 'iconic_ww_statuses', array(
			'publish' => array(
				'label'       => __( 'Public', 'iconic-ww' ),
				'description' => __( 'Anyone can search for and see this wishlist. You can also use the sharing link.', 'iconic-ww' ),
			),
			'private' => array(
				'label'       => __( 'Private', 'iconic-ww' ),
				'description' => __( 'Only you can see this wishlist.', 'iconic-ww' ),
			),
		) );
	}

	/**
	 * Get message for when there are no wishlists to see.
	 *
	 * @param string|bool $endpoint
	 *
	 * @return bool|string
	 */
	public static function get_no_wishlists_message( $endpoint = false ) {
		$endpoint = $endpoint ? $endpoint : Iconic_WW_Endpoints::$current_endpoint;
		$message  = false;
		$create   = false;

		if ( $endpoint === 'base' ) {
			$message = __( 'You have not created any wishlists yet.', 'iconic-ww' );
			$create  = true;
		} elseif ( $endpoint === 'find' ) {
			$s = filter_input( INPUT_GET, 'search' );

			if ( $s ) {
				$message = sprintf( __( 'There are no wishlists to see for "%s".', 'iconic-ww' ), $s );
			} else {
				$message = __( 'There are no wishlists to see.', 'iconic-ww' );
			}
		}

		if ( $create ) {
			$message .= sprintf( ' <a href="javascript: void( 0 );" class="button" data-iconic_ww_action="get_create_wishlist_modal">%s</a>', __( 'Create a wishlist', 'iconic-ww' ) );
		}

		return sprintf( '<p class="woocommerce-info">%s</p>', $message );
	}

	/**
	 * Add admin columns.
	 *
	 * @param array $columns
	 *
	 * @return array
	 */
	public static function admin_columns( $columns ) {
		unset( $columns['date'], $columns['author'] );

		$columns['iconic_ww_items']   = __( 'Items', 'iconic-ww' );
		$columns['iconic_ww_privacy'] = __( 'Privacy', 'iconic-ww' );
		$columns['author']            = __( 'Author' );
		$columns['date']              = __( 'Date' );

		return $columns;
	}

	/**
	 * Admin columns content.
	 *
	 * @param string $column
	 * @param int    $post_id
	 */
	public static function admin_columns_content( $column, $post_id ) {
		$wishlist = new Iconic_WW_Wishlist( $post_id );

		switch ( $column ) {
			case 'iconic_ww_items' :
				echo $wishlist->get( 'items_count' );
				break;
			case 'iconic_ww_privacy' :
				echo $wishlist->get( 'privacy' );
				break;
		}
	}

	/**
	 * Add Meta boxes.
	 */
	public static function meta_boxes() {
		add_meta_box(
			'iconic-ww-items',
			sprintf( __( '%s Items', 'iconic-ww' ), self::get_wishlists_label( 'singular' ) ),
			array( __CLASS__, 'meta_box_items' ),
			self::$post_type
		);
	}

	/**
	 * Display meta box items.
	 *
	 * @param WP_Post|null $post
	 */
	public static function meta_box_items( $post ) {
		if ( ! $post ) {
			?>
			<p><?php _e( 'This wishlist is currently empty.', 'iconic-ww' ); ?></p>
			<?php
			return;
		}

		$wishlist = new Iconic_WW_Wishlist( $post );
		$items    = $wishlist->get( 'items' );

		if ( empty( $items ) ) {
			?>
			<p><?php _e( 'This wishlist is currently empty.', 'iconic-ww' ); ?></p>
			<?php
			return;
		}
		?>
		<style>
			.iconic-ww-items {
				margin: 15px 0 0;
			}

			.iconic-ww-items img {
				max-width: 100%;
				height: auto;
			}

			.iconic-ww-items__item td {
				vertical-align: middle;
			}

			.iconic-ww-items__item-image-column {
				width: 50px;
			}
		</style>
		<table class="iconic-ww-items wp-list-table widefat fixed striped">
			<thead>
			<tr>
				<th class="iconic-ww-items__item-image-column">&nbsp;</th>
				<th><?php _e( 'Product', 'iconic-ww' ); ?></th>
				<th><?php _e( 'Added', 'iconic-ww' ); ?></th>
			</tr>
			</thead>
			<tbody>
			<?php foreach ( $items as $item ) { ?>
				<tr class="iconic-ww-items__item">
					<td class="iconic-ww-items__item-image-column"><?php echo $item['product']->get_image( 'thumbnail' ); ?></td>
					<td>
						<a href="<?php echo esc_attr( admin_url( 'post.php?post=' . $item['product']->get_id() . '&action=edit' ) ); ?>"><?php echo $item['product']->get_title(); ?></a>
					</td>
					<td><?php echo $item['added']; ?></td>
				</tr>
			<?php } ?>
			</tbody>
		</table>
		<?php
	}

	/**
	 * Add filter to author dropdown.
	 */
	public static function load_user_dropdown_filter() {
		$screen = get_current_screen();

		if ( empty( $screen->post_type ) || self::$post_type !== $screen->post_type ) {
			return;
		}

		add_filter( 'wp_dropdown_users_args', array( __CLASS__, 'dropdown_users_args' ), 10, 2 );
	}

	/**
	 * Add roles to "author" dropdown.
	 *
	 * @param $args
	 * @param $r
	 *
	 * @return mixed
	 */
	public static function dropdown_users_args( $args, $r ) {
		global $post;

		// Check that this is the correct drop-down.
		if ( 'post_author_override' === $r['name'] && self::$post_type === $post->post_type ) {
			$roles = self::get_roles();

			// If we have roles, change the args to only get users of those roles.
			if ( $roles ) {
				$args['who']      = '';
				$args['role__in'] = $roles;
			}
		}

		return $args;
	}

	/**
	 * Get roles for post type.
	 *
	 * @return array
	 */
	public static function get_roles() {
		global $wp_roles;

		$roles = array();
		$type  = get_post_type_object( self::$post_type );

		// Get the post type object caps.
		$caps = array( $type->cap->edit_posts, $type->cap->publish_posts, $type->cap->create_posts );
		$caps = array_unique( $caps );

		// Loop through the available roles.
		foreach ( $wp_roles->roles as $name => $role ) {
			foreach ( $caps as $cap ) {
				// If the role is granted the cap, add it.
				if ( isset( $role['capabilities'][ $cap ] ) && true === $role['capabilities'][ $cap ] ) {
					$roles[] = $name;
					break;
				}
			}
		}

		$roles[] = 'customer';

		return array_filter( $roles );
	}
}